require "bd_pod_cache_lock/version"
require 'fileutils'
require 'tmpdir'
require 'digest/sha1'

ROOT_PROCESS_ID = Process.pid
$locked = false

$pod_cache_dir = Pod::Config.instance.cache_root + 'Pods'
$pod_cache_lock_dir = $pod_cache_dir + 'CacheLock'
$pod_install_lock_file = $pod_cache_dir + 'pod_install.lock'

at_exit do
  if(Process.pid == ROOT_PROCESS_ID && $locked)
    FileUtils.rm_rf($pod_install_lock_file)
    puts "UnLock Pod install"
  end
end

module Pod
  class Installer
    # Installs the Pods.
    #
    # The installation process is mostly linear with a few minor complications
    # to keep in mind:
    #
    # - The stored podspecs need to be cleaned before the resolution step
    #   otherwise the sandbox might return an old podspec and not download
    #   the new one from an external source.
    # - The resolver might trigger the download of Pods from external sources
    #   necessary to retrieve their podspec (unless it is instructed not to
    #   do it).
    #
    # @return [void]
    #
    def install!
      Dir.mkdir($pod_cache_dir) unless Dir.exist?($pod_cache_dir)

      lock_file_path = $pod_install_lock_file

      File.new(lock_file_path, File::CREAT) unless File.exist? lock_file_path
      File.open(lock_file_path) do |file|
        flock(file, File::LOCK_EX) { 
          puts "Lock Pod install"
          $locked = true
          prepare
          resolve_dependencies
          download_dependencies
        }
      end
      
      validate_targets
      generate_pods_project
      if installation_options.integrate_targets?
        integrate_user_project
      else
        UI.section 'Skipping User Project Integration'
      end
      perform_post_install_actions
    end

    # @param  [File] file
    #         should be locked file.
    #
    # @param  [mode] mode
    #         the file lock mode.
    #
    # @return [Boolean] The result of lock specific file and mode.
    #
    def flock(file, mode)
      success = file.flock(mode)
      if success
        begin
          yield file
        ensure
          file.flock(File::LOCK_UN)
        end
      end
      success
    end
  end
end

